import { kComb } from "./internal/util.js";
import { map } from "./transformers.js";
/**
 * Calls the specified callback function for all the items in an iterable. The
 * return value of the callback function is the accumulated result, and is
 * provided as an argument in the next call to the callback function.
 * @param it The iterable to be reduced.
 * @param {ReduceAccumulatorCallback} reducer A function that accepts up to
 * four arguments. The reduce function calls `f` one time for each item in the
 * iterable, unless it is stopped early.
 * @param initialValue If `initialValue` is specified, it is used as the initial
 * value to start accumulation. The first call to `f` provides this value as an
 * argument.
 * @param {ReduceStopCallback} stop A functin similar to `f`. If it returns false,
 * `reduce` will stop early and return the current accumulated value.
 * @typeParam T - Type of items in `it`.
 * @typeParam U - Type of accumulator and result.
 * @returns The final accumulator value.
 * @example
 * ```ts
 * import * as iter from "https://deno.land/x/iter";
 *
 * const naturals = iter.create.increments(1);
 * // The first triangular number which is a multiple of 13
 * const luckyNumber = iter.reduce(
 *   naturals,
 *   (tot, n) => tot + n,
 *   0,
 *   (tot) => tot % 13 === 0,
 * );
 *
 * console.log(luckyNumber); // -> 78
 * ```
 */
export function reduce(it, reducer, initialValue, // TODO: Overload to make this optional
stop = kComb(false)) {
    let index = 0;
    let accumulator = initialValue;
    for (const item of it) {
        accumulator = reducer(accumulator, item, index, it);
        if (stop(accumulator, item, index, it))
            break;
        index++;
    }
    return accumulator;
}
/**
 * Determines whether the specified callback function returns true for any item
 * in an iterable.
 *
 * :warning: When ran on an endless iterable for which `predicate` never returns
 * true, this functions will never return.
 * @param it - The iterable to be tested.
 * @param {IterablePredicateCallback} predicate A function that accepts up to
 * three arguments. The some function calls the predicate function for each
 * element in the array until the predicate returns a value
 * @typeParam T - The type of items in `it`.
 * @returns Whether any of the items in `it` predicate true.
 * @example
 * ```ts
 * import * as iter from "https://deno.land/x/iter";
 *
 * const naturals = iter.create.increments(1);
 * const hasEvens = iter.some(naturals, (n) => n % 2 === 0);
 *
 * console.log(hasEvens); // -> true
 *
 * // Will never return.
 * // const hasNeg = iter.some(naturals, (n) => n < 0);
 * ```
 */
export function some(it, predicate) {
    const predicateResults = map(it, predicate);
    return reduce(predicateResults, (acc, result) => acc || result, false, (acc) => acc);
}
/**
 * Determines whether the specified callback function returns true for all items
 * in an iterable.
 *
 * :warning: When ran on an endless iterable for which `predicate` never returns
 * false, this function will never return.
 * @param it - The iterable to be tested.
 * @param {IterablePredicateCallback} predicate A function that accepts up to
 * three arguments. The some function calls the predicate function for each
 * element in the array until the predicate returns a value
 * @typeParam T - The type of items in `it`.
 * @returns Whether any of the items in `it` predicate true.
 * @example
 * ```ts
 * import * as iter from "https://deno.land/x/iter";
 *
 * const naturals = iter.create.increments(1);
 * const allOdd = iter.every(naturals, (n) => n % 2 === 1);
 *
 * console.log(allOdd); // -> false
 *
 * // Will never return.
 * // const allPositive = iter.every(naturals, (n) => n > 0);
 * ```
 */
export function every(it, predicate) {
    const predicateResults = map(it, predicate);
    return reduce(predicateResults, (acc, result) => acc && result, true, (acc) => !acc);
}
/**
 * Determines whether an iterable includes a certain element, returning true or
 * false as appropriate.
 *
 * :warning: When ran on an endless iterable which does not contain `value`,
 * this function never returns.
 * @param it - The iterable to be tested.
 * @param value - The item to search for.
 * @typeParam T - The type of items in `it`.
 * @returns Whether `value` is in `it`.
 * @example
 * ```ts
 * import * as iter from "https://deno.land/x/iter";
 *
 * const naturals = iter.create.increments(1);
 * const has100 = iter.includes(naturals, 100);
 *
 * console.log(has100); // -> true
 *
 * // Will never return.
 * // const has0 = iter.includes(naturals, 0);
 * ```
 */
export function includes(it, value) {
    for (const item of it) {
        if (item === value)
            return true;
    }
    return false;
}
/**
 * Returns the value of the first item in the iterable where predicate is true,
 * and undefined otherwise.
 * @param it - The iterable to search.
 * @param {IterablePredicateCallback} predicate - Find calls predicate once for
 * each item in the iterable, in ascending order, until it finds one where
 * predicate returns true. If such an item is found, find immediately returns
 * that element value, Otherwise, find returns undefined.
 * @typeParams T - The type of items in `it`.
 * @returns The first item which satisfied `predicate`.
 * @example
 * ```ts
 * import * as iter from "https://deno.land/x/iter";
 *
 * const naturals = iter.create.increments(1);
 * // Find a solution to 20n = 2n² - 6n, n ∈ ℕ
 * const solution1 = iter.find(naturals, (n) => 20 * n === 2 * n ** 2 - 6 * n);
 *
 * console.log(solution1); // -> 13
 *
 * // Will never return.
 * // Find a solution to n³ = 3n, n ∈ ℕ
 * // const solution2 = iter.find(naturals, (n) => n ** 3 === 3 * n);
 */
export function find(it, predicate) {
    let index = 0;
    for (const item of it) {
        if (predicate(item, index++, it))
            return item;
    }
    return undefined;
}
/**
 * Returns the index of the first item in the iterable where predicate is true,
 * and -1 otherwise.
 * @param it - The iterable to search.
 * @param {IterablePredicateCallback} predicate find calls predicate once for
 * each item in the iterable, in ascending order, until it finds one where
 * predicate returns true. If such an item is found, findIndex immediately
 * returns that element index. Otherwise, findIndex returns -1.
 * @typeParams T - The type of items in `it`.
 * @returns The first item which satisfied `predicate`.
 * @example
 * ```ts
 * import * as iter from "https://deno.land/x/iter";
 *
 * const naturals = iter.create.increments(1);
 * // Find a solution to 20n = 2n² - 6n, n ∈ ℕ
 * const solutionIndex1 = iter.findIndex(
 *   naturals,
 *   (n) => 20 * n === 2 * n ** 2 - 6 * n,
 * );
 *
 * console.log(solutionIndex1); // -> false
 *
 * // Will never return.
 * // Find a solution to n³ = 3n, n ∈ ℕ
 * const solutionIndex2 = iter.findIndex(naturals, (n) => n ** 3 === 3 * n);
 *
 * const first10 = iter.take(naturals, 10);
 * const pos = iter.findIndex(first10, x => x < 0);
 *
 * console.log(pos); // -> -1
 * ```
 */
export function findIndex(it, predicate) {
    let index = 0;
    for (const item of it) {
        if (predicate(item, index, it))
            return index;
        index++;
    }
    return -1;
}
/**
 * Finds the sum of all items in `it`.
 *
 * :warning: When ran on an endless iterable, this never returns.
 * @param it - The iterable to sum.
 * @returns The sum of all items in `it`.
 * @example
 * ```ts
 * import * as iter from "https://deno.land/x/iter";
 *
 * const myRange = iter.create.range(1, 100);
 * console.log(iter.sum(myRange)); // -> 5050
 * ```
 */
export function sum(it) {
    return reduce(it, (acc, n) => acc + n, 0, (acc) => isNaN(acc));
}
/**
 * Finds the product of all items in `it`.
 *
 * :warning: When ran on an endless iterable without any zeros, this never
 * returns.
 * @param it - The iterable to sum.
 * @returns The product of all items in `it`.
 * @example
 * ```ts
 * import * as iter from "https://deno.land/x/iter";
 *
 * const myRange = iter.create.range(1, 10);
 * console.log(iter.product(myRange)); // -> 3628800
 *
 * // Demonstrating laziness
 * const rangeWithZero = iter.create.range(-1, 10);
 * const observerdIter = iter.lazyObserver(
 *   rangeWithZero,
 *   (x) => console.log("Processed: " + x),
 * );
 * // -> Processed: -1
 * // -> Processed: 0
 * console.log(iter.product(observerdIter)); // -> -0
 * ```
 */
export function product(it) {
    return reduce(it, (acc, n) => acc * n, 1, (acc) => acc === 0 || isNaN(acc));
}
/**
 * Finds the norm of `it` (distance from the origin).
 *
 * :warning: When ran on an endless iterable, this never returns.
 * @param it - The iterable to sum.
 * @returns The norm of `it`.
 * @example
 * ```ts
 * import * as iter from "https://deno.land/x/iter";
 *
 * const pythagoreadQuad = [12, 16, 21];
 * console.log(iter.norm(pythagoreadQuad)); // -> 29
 * ```
 */
export function norm(it) {
    return Math.sqrt(reduce(it, (acc, n) => acc + n ** 2, 0, (acc) => isNaN(acc)));
}
