"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.lazyObserver = exports.forEach = void 0;
/**
 * Performs the specified action for each item in an iterable, consuming the
 * iterable in the process.
 * @param it - The iterable being looped over.
 * @param {ForEachCallback} f - A function that accepts up to three arguments.
 * `forEach` calls `f` one time for each item in the iterable.
 * @typeParam T - The type of items in `it`.
 * @example
 * ```ts
 * import * as iter from "https://deno.land/x/iter/mod.ts";
 *
 * const naturals = iter.create.increments(1);
 * const first6 = iter.take(naturals, 6);
 * iter.forEach(first6, x => console.log(x));
 *
 * // -> 1
 * // -> 2
 * // -> 3
 * // -> 4
 * // -> 5
 * // -> 6
 * ```
 */
function forEach(it, f) {
    let index = 0;
    for (const item of it)
        f(item, index++, it);
}
exports.forEach = forEach;
/**
 * Performs the specified action for each item in an iterable when the returned
 * iterable is iterated over. Like {@link forEach | `forEach`} but non-consuming.
 *
 * Can be used to observe items of an iterable as they are released.
 * @param it - The iterable being observed.
 * @param f - A function that accepts up to three arguments. `forEach` calls `f`
 * one time for each item in the iterable.
 * @typeParam T - The type of items in `it`.
 * @returns - The observed version of `it`.
 * @example
 * ```ts
 * import * as iter from "https://deno.land/x/iter/mod.ts";
 *
 * const naturals = iter.create.increments(1);
 * const observed = iter.lazyObserver(naturals, (x) => console.log(x));
 * const iterator = observed[Symbol.iterator]();
 *
 * iterator.next();
 * // -> 1
 * iterator.next();
 * // -> 2
 * iterator.next();
 * // -> 3
 * ```
 */
function lazyObserver(it, f) {
    return {
        *[Symbol.iterator]() {
            let index = 0;
            for (const item of it) {
                f(item, index++, it);
                yield item;
            }
        },
    };
}
exports.lazyObserver = lazyObserver;
